@@ -22,6 +22,7 @@
  */
 
 #include "vncviewer.h"
+#include <ctype.h>
 #include <X11/Xaw/Toggle.h>
 
 char *programName;
@@ -31,6 +32,7 @@ Display* dpy;
 Widget toplevel;
 
 extern void raiseme(int force);
+extern void CreateChat(void);
 
 void set_sbwidth(int sbw) {
 	char *q, *p, t[5];
@@ -68,6 +70,20 @@ void set_sbwidth(int sbw) {
 	}
 }
 
+void min_title(void) {
+	char *q;
+	int k;
+
+	k = 0;
+	while (fallback_resources[k] != NULL) {
+		q = strstr(fallback_resources[k], "Ssvnc.title: ");
+		if (q) {
+			fallback_resources[k] = strdup("Ssvnc.title: %s");
+		}
+		k++;
+	}
+}
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -248,15 +264,21 @@ static void chat_window_only(void) {
 	}
 }
 
+int saw_appshare = 0;
+
 int
 main(int argc, char **argv)
 {
-	int i, save_sbw;
+	int i, save_sbw, saw_listen = 0;
 	char *pw_loc = NULL;
 	programName = argv[0];
 
+	if (strrchr(programName, '/') != NULL) {
+		programName = strrchr(programName, '/') + 1;
+	}
+
 	for (i = 1; i < argc; i++) {
-		if (strcmp(argv[i], "-env") == 0) {
+		if (!strcmp(argv[i], "-env")) {
 			if (i+1 < argc) {
 				char *estr = argv[i+1];
 				if (strchr(estr, '=')) {
@@ -264,6 +286,18 @@ main(int argc, char **argv)
 				}
 			}
 		}
+		if (!strcmp(argv[i], "-noipv4")) {
+			putenv("VNCVIEWER_NO_IPV4=1");
+		}
+		if (!strcmp(argv[i], "-noipv6")) {
+			putenv("VNCVIEWER_NO_IPV6=1");
+		}
+	}
+	if (getenv("VNCVIEWER_NO_IPV4")) {
+		appData.noipv4 = True;
+	}
+	if (getenv("VNCVIEWER_NO_IPV6")) {
+		appData.noipv6 = True;
 	}
 
   /* The -listen option is used to make us a daemon process which listens for
@@ -277,36 +311,49 @@ main(int argc, char **argv)
      flag. */
 
 	for (i = 1; i < argc; i++) {
+		if (!strcmp(argv[i], "-appshare")) {
+			putenv("SSVNC_MULTIPLE_LISTEN=1");
+			fprintf(stderr, "Enabling -multilisten mode for 'x11vnc -appshare' usage.\n\n");
+			saw_appshare = 1;
+		}
+		if (!strcmp(argv[i], "-multilisten")) {
+			putenv("SSVNC_MULTIPLE_LISTEN=1");
+			saw_listen = 2;
+		}
+		if (!strcmp(argv[i], "-listen")) {
+			saw_listen = 1;
+		}
+		if (!strcmp(argv[i], "-acceptpopup")) {
+			putenv("SSVNC_ACCEPT_POPUP=1");
+		}
+		if (!strcmp(argv[i], "-acceptpopupsc")) {
+			putenv("SSVNC_ACCEPT_POPUP_SC=1");
+		}
 		if (strstr(argv[i], " pw=") != NULL) {
 			pw_loc = strstr(argv[i], " pw=") + 1;
 		}
 	}
 
 	for (i = 1; i < argc; i++) {
-		if (strcmp(argv[i], "-acceptpopup") == 0) {
-			putenv("SSVNC_ACCEPT_POPUP=1");
-			continue;
-		}
-		if (strcmp(argv[i], "-acceptpopupsc") == 0) {
-			putenv("SSVNC_ACCEPT_POPUP_SC=1");
-			continue;
-		}
-		if (strcmp(argv[i], "-listen") == 0) {
+		if (!strcmp(argv[i], "-appshare") && !saw_listen) {
 			listenForIncomingConnections(&argc, argv, i);
 			break;
 		}
-		if (strcmp(argv[i], "-multilisten") == 0) {
-			putenv("SSVNC_MULTIPLE_LISTEN=1");
+		if (!strcmp(argv[i], "-multilisten")) {
 			listenForIncomingConnections(&argc, argv, i);
 			break;
 		}
-		if (strcmp(argv[i], "-tunnel") == 0 || strcmp(argv[i], "-via") == 0) {
+		if (!strcmp(argv[i], "-listen")) {
+			listenForIncomingConnections(&argc, argv, i);
+			break;
+		}
+		if (!strcmp(argv[i], "-tunnel") || !strcmp(argv[i], "-via")) {
 			if (!createTunnel(&argc, argv, i)) {
 				exit(1);
 			}
 			break;
 		}
-		if (strcmp(argv[i], "-printres") == 0 || strcmp(argv[i], "-res") == 0) {
+		if (!strcmp(argv[i], "-printres") || !strcmp(argv[i], "-res")) {
 			int j = 0;
 			fprintf(stdout, "\n! Ssvnc fallback X resources:\n\n");
 			while (1) {
@@ -336,6 +383,9 @@ main(int argc, char **argv)
      generating appropriate resource specs, and makes a connection to the X
      display. */
 
+	if (saw_appshare || getenv("VNCVIEWER_MIN_TITLE")) {
+		min_title();
+	}
 	appData.sbWidth = 0;
 	if (getenv("VNCVIEWER_SBWIDTH")) {
 		int sbw = atoi(getenv("VNCVIEWER_SBWIDTH"));
@@ -374,6 +424,10 @@ main(int argc, char **argv)
 
 	GetArgsAndResources(argc, argv);
 
+	if (saw_appshare) {
+		appData.appShare = True;
+	}
+
 	if (save_sbw) {
 		appData.sbWidth = save_sbw;
 	}
@@ -419,7 +473,7 @@ main(int argc, char **argv)
 						tmp[i] = '\0';
 					}
 					for (i=0; i<250; i++) {
-						if (i >= strlen(appData.repeaterUltra)) {
+						if (i >= (int) strlen(appData.repeaterUltra)) {
 							break;
 						}
 						tmp[i] = appData.repeaterUltra[i];
@@ -519,7 +573,7 @@ static Widget bfull = NULL;
 
 int do_format_change = 0;
 int do_cursor_change = 0;
-int do_fb_update = 0.0;
+double do_fb_update = 0.0;
 static void schedule_format_change(void) {
 	do_format_change = 1;
 	do_cursor_change = 0;
@@ -572,6 +626,7 @@ Toggle8bpp(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		fprintf(stderr, "8bpp: on (%d colors)\n", appDataNew.useBGR233);
 	}
 	schedule_format_change();
+	if (w || ev || params || num_params) {}
 }
 
 
@@ -592,6 +647,7 @@ Toggle16bpp(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		fprintf(stderr, "16bpp: on\n");
 	}
 	schedule_format_change();
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -619,6 +675,7 @@ ToggleFullColor(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		}
 	}
 	schedule_format_change();
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -632,6 +689,7 @@ ToggleXGrab(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	fprintf(stderr, "ToggleXGrab, current=%d\n", appData.grabAll);
 	/* always ungrab to be sure, fullscreen will handle the rest */
 	XUngrabServer(dpy);
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -642,6 +700,7 @@ ToggleEscapeActive(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		appData.escapeActive = True;
 	}
+	if (w || ev || params || num_params) {}
 }
 
 /*
@@ -664,6 +723,7 @@ Toggle256Colors(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		appDataNew.useBGR565 = False;
 		schedule_format_change();
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -678,6 +738,7 @@ Toggle64Colors(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		appDataNew.useBGR565 = False;
 		schedule_format_change();
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -692,6 +753,7 @@ Toggle8Colors(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		appDataNew.useBGR565 = False;
 		schedule_format_change();
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -707,6 +769,7 @@ ToggleGreyScale(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		fprintf(stderr, "greyscale: on\n");
 	}
 	schedule_format_change();
+	if (w || ev || params || num_params) {}
 }
 
 /*
@@ -725,6 +788,7 @@ ToggleJPEG(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		fprintf(stderr, "JPEG: on\n");
 	}
 	schedule_format_change();
+	if (w || ev || params || num_params) {}
 }
 
 /*
@@ -782,6 +846,7 @@ ToggleTightZRLE(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		first = 0;
 	}
 	schedule_format_change();
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -812,6 +877,7 @@ ToggleZRLEZYWRLE(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		}
 	}
 	schedule_format_change();
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -858,6 +924,7 @@ ToggleTightHextile(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		first = 0;
 	}
 	schedule_format_change();
+	if (w || ev || params || num_params) {}
 }
 
 void scale_check_zrle(void) {
@@ -870,7 +937,7 @@ void scale_check_zrle(void) {
 		return;
 	}
 	if (!usingZRLE && !usingHextile) {
-		Widget w;
+		Widget w = 0;
 		fprintf(stderr, "\nSwitching to faster ZRLE encoding in client-side scaling mode.\n");
 		fprintf(stderr, "Switch back to Tight via the Popup menu if you prefer it.\n\n");
 		ToggleTightZRLE(w, NULL, NULL, NULL);
@@ -892,6 +959,7 @@ ToggleViewOnly(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		fprintf(stderr, "viewonly: on\n");
 	}
 	Xcursors(1);
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -911,6 +979,7 @@ ToggleCursorShape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		do_cursor_change = -1;
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -923,6 +992,7 @@ ToggleCursorAlpha(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		appData.useCursorAlpha = True;
 		fprintf(stderr, "useCursorAlpha: on\n");
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -938,6 +1008,7 @@ ToggleX11Cursor(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	}
 	schedule_format_change();
 	do_cursor_change = 1;
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -950,6 +1021,7 @@ ToggleBell(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		appData.useBell = True;
 		fprintf(stderr, "useBell: on\n");
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -964,6 +1036,7 @@ ToggleRawLocal(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		fprintf(stderr, "useRawLocal: on\n");
 	}
 	schedule_format_change();
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -978,6 +1051,7 @@ ToggleServerInput(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		fprintf(stderr, "serverInput: on\n");
 		SendServerInput(False);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -991,6 +1065,7 @@ TogglePipelineUpdates(Widget w, XEvent *ev, String *params, Cardinal *num_params
 		fprintf(stderr, "pipeline-update: on\n");
 	}
 	/* XXX request one to be sure? */
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1003,6 +1078,7 @@ ToggleSendClipboard(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		appData.sendClipboard = True;
 		fprintf(stderr, "Send CLIPBOARD Selection: on (do not send PRIMARY)\n");
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1015,6 +1091,7 @@ ToggleSendAlways(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		appData.sendAlways = True;
 		fprintf(stderr, "Send Selection Always: on\n");
 	}
+	if (w || ev || params || num_params) {}
 }
 
 
@@ -1040,11 +1117,17 @@ ToggleSingleWindow(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 			XDefineCursor(dpy, desktopWin, bogoCursor);
 		}
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void raiseme(int force);
 void AppendChatInput(char *);
 
+extern void ShowChat(Widget w, XEvent *event, String *params, Cardinal *num_params);
+extern void ShowFile(Widget w, XEvent *event, String *params, Cardinal *num_params);
+extern Bool SendTextChatFinished(void);
+
+
 void printChat(char *str, Bool raise) {
 	if (appData.termChat) {
 		if (raise) {
@@ -1053,7 +1136,7 @@ void printChat(char *str, Bool raise) {
 		fprintf(stderr, str);
 	} else {
 		if (raise) {
-			ShowChat();
+			ShowChat(0, 0, 0, 0);
 		}
 		AppendChatInput(str);
 	}
@@ -1069,7 +1152,7 @@ ToggleTextChat(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		HideChat(0, NULL, NULL, NULL);
 		appData.chatActive= False;
 	} else {
-		ShowChat();
+		ShowChat(0, 0, 0, 0);
 		SendTextChatOpen();
 		if (appData.termChat) {
 			printChat("\n*SentOpen*\n\nSend: ", True);
@@ -1078,6 +1161,7 @@ ToggleTextChat(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		}
 		appData.chatActive = True;
 	}
+	if (w || ev || params || num_params) {}
 }
 
 extern int filexfer_sock;
@@ -1092,8 +1176,10 @@ ToggleFileXfer(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
 	static double last_start = 0.0;
 	if (appData.fileActive) {
-		//HideFile(w, ev, params, num_params);
-		//appData.fileActive = False;
+#if 0
+		HideFile(w, ev, params, num_params);
+		appData.fileActive = False;
+#endif
 #ifndef KILLJAVA
 		if (filexfer_sock >= 0) {
 			close(filexfer_sock);
@@ -1120,9 +1206,11 @@ ToggleFileXfer(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		appData.fileActive = True;
 		last_start = dnow();
 	}
+	if (w || ev || params || num_params) {}
 }
 
 static int fooHandler(Display *dpy, XErrorEvent *error) {
+	if (dpy || error) {}
 	return 0;
 }
 
@@ -1176,6 +1264,7 @@ DoServerScale(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		n = atoi(s);
 		set_server_scale(n);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void set_server_quality(int n) {
@@ -1210,6 +1299,7 @@ DoServerQuality(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		n = atoi(s);
 		set_server_quality(n);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void set_server_compress(int n) {
@@ -1244,9 +1334,11 @@ DoServerCompress(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		n = atoi(s);
 		set_server_compress(n);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 extern void rescale_image(void);
+extern void get_scale_values(double *fx, double *fy);
 
 void
 SetScale(Widget w, XEvent *ev, String *params, Cardinal *num_params)
@@ -1254,8 +1346,10 @@ SetScale(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	char *s;
 	s = DoScaleDialog();
 	if (s[0] != '\0') {
+#if 0
 		int w = si.framebufferWidth;
 		int h = si.framebufferHeight;
+#endif
 		double fx, fy;
 		int fs = 0;
 		if (appData.scale != NULL && !strcmp(s, appData.scale)) {
@@ -1288,6 +1382,7 @@ SetScale(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 			FullScreenOn();
 		}
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1299,6 +1394,7 @@ SetEscapeKeys(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	if (s[0] != '\0') {
 		appData.escapeKeys = strdup(s);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void set_ycrop(int n) {
@@ -1332,6 +1428,7 @@ SetYCrop(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		n = atoi(s);
 		set_ycrop(n);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void set_scbar(int n) {
@@ -1366,6 +1463,7 @@ SetScbar(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		n = atoi(s);
 		set_scbar(n);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1375,6 +1473,7 @@ SetScaleN(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		int n = atoi(params[0]);
 		set_server_scale(n);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void UpdateQual(void) {
@@ -1488,6 +1587,7 @@ SetTurboVNC(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 			LosslessRefresh();
 		}
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1497,6 +1597,7 @@ SetQuality(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		int n = atoi(params[0]);
 		set_server_quality(n);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1506,6 +1607,7 @@ SetCompress(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 		int n = atoi(params[0]);
 		set_server_compress(n);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1555,6 +1657,7 @@ SetViewOnlyState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1565,6 +1668,7 @@ SetNOJPEGState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, True, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1578,6 +1682,7 @@ SetQualityState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 			XtVaSetValues(w, XtNstate, False, NULL);
 		}
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1591,6 +1696,7 @@ SetCompressState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 			XtVaSetValues(w, XtNstate, False, NULL);
 		}
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1604,6 +1710,7 @@ SetScaleNState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 			XtVaSetValues(w, XtNstate, False, NULL);
 		}
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1620,6 +1727,7 @@ Set8bppState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1636,6 +1744,7 @@ Set16bppState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1652,6 +1761,7 @@ SetFullColorState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 			XtVaSetValues(b16, XtNstate, False, NULL);
 		}
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1662,6 +1772,7 @@ SetXGrabState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1672,6 +1783,7 @@ SetEscapeKeysState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1688,6 +1800,7 @@ Set256ColorsState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1704,6 +1817,7 @@ Set64ColorsState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1720,6 +1834,7 @@ Set8ColorsState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1730,9 +1845,10 @@ SetGreyScaleState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
-static init_state(void) {
+static void init_state(void) {
 	static int first = 1;
 	if (first && appData.encodingsString) {
 		char *t, *z, *y, *h;
@@ -1777,6 +1893,7 @@ SetZRLEState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1788,6 +1905,7 @@ SetHextileState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1799,6 +1917,7 @@ SetZYWRLEState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1809,6 +1928,7 @@ SetCursorShapeState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1819,6 +1939,7 @@ SetCursorAlphaState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1829,6 +1950,7 @@ SetX11CursorState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1839,6 +1961,7 @@ SetBellState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, True, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1849,6 +1972,7 @@ SetRawLocalState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1859,6 +1983,7 @@ SetServerInputState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1869,6 +1994,7 @@ SetPipelineUpdates(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, True, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1879,6 +2005,7 @@ SetSendClipboard(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, True, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1889,6 +2016,7 @@ SetSendAlways(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, True, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1899,6 +2027,7 @@ SetSingleWindowState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1909,6 +2038,7 @@ SetTextChatState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
 
 void
@@ -1919,4 +2049,5 @@ SetFileXferState(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	} else {
 		XtVaSetValues(w, XtNstate, False, NULL);
 	}
+	if (w || ev || params || num_params) {}
 }
