@@ -74,9 +74,11 @@ extern int endianTest;
 #define TVNC_SAMPOPT 4
 enum {TVNC_1X=0, TVNC_4X, TVNC_2X, TVNC_GRAY};
 
+#if 0
 static const char *subsampLevel2str[TVNC_SAMPOPT] = {
   "1X", "4X", "2X", "Gray"
 };
+#endif
 #ifdef TURBOVNC
 #define rfbTightNoZlib 0x0A
 #define rfbTurboVncVendor "TRBO"
@@ -89,7 +91,9 @@ static const char *subsampLevel2str[TVNC_SAMPOPT] = {
 #endif
 
 /* for debugging width, height, etc */
-//#define XtVaSetValues printf("%s:%d\n", __FILE__, __LINE__); XtVaSetValues
+#if 0
+#define XtVaSetValues printf("%s:%d\n", __FILE__, __LINE__); XtVaSetValues
+#endif
 
 
 /* argsresources.c */
@@ -168,6 +172,7 @@ typedef struct {
 
 	char *scale;
 	char *escapeKeys;
+	Bool appShare;
 	Bool escapeActive;
 	Bool pipelineUpdates;
 
@@ -180,6 +185,9 @@ typedef struct {
 	int subsampLevel;
 	Bool doubleBuffer;
 
+	Bool noipv4;
+	Bool noipv6;
+
 } AppData;
 
 extern AppData appData;
@@ -228,6 +236,10 @@ extern GC gc;
 extern GC srcGC, dstGC;
 extern Dimension dpyWidth, dpyHeight;
 
+extern int appshare_0_hint;
+extern int appshare_x_hint;
+extern int appshare_y_hint;
+
 extern void DesktopInitBeforeRealization();
 extern void DesktopInitAfterRealization();
 extern void Xcursors(int set);
@@ -404,6 +416,7 @@ extern void SelectionFromVNC(Widget w, XEvent *event, String *params,
 
 extern XImage *CreateShmImage(int do_ycrop);
 extern void ShmCleanup();
+extern void ShmDetach();
 extern Bool UsingShm();
 
 /* sockets.c */
@@ -414,9 +427,12 @@ extern Bool ReadFromRFBServer(char *out, unsigned int n);
 extern Bool WriteExact(int sock, char *buf, int n);
 extern int FindFreeTcpPort(void);
 extern int ListenAtTcpPort(int port);
-extern int ConnectToTcpAddr(unsigned int host, int port);
+extern int ListenAtTcpPort6(int port);
+extern int dotted_ip(char *host, int partial);
+extern int ConnectToTcpAddr(const char *hostname, int port);
 extern int ConnectToUnixSocket(char *file);
 extern int AcceptTcpConnection(int listenSock);
+extern int AcceptTcpConnection6(int listenSock);
 extern Bool SetNonBlocking(int sock);
 extern Bool SetNoDelay(int sock);
 extern Bool SocketPair(int fd[2]);
